import { Drink, DrinkPriceVolume, usePricelistStore } from 'src/plugins/pricelist/store';

function calc_volume(volume: DrinkPriceVolume) {
  if (volume.ingredients.some((ingredient) => !!ingredient.drink_ingredient)) {
    let retVal = 0;
    volume.ingredients.forEach((ingredient) => {
      if (ingredient.drink_ingredient?.volume) {
        retVal += ingredient.drink_ingredient.volume;
      }
    });
    return retVal;
  } else {
    return volume._volume;
  }
}

function calc_cost_per_volume(drink: Drink) {
  let retVal = drink._cost_per_volume;
  if (!!drink.volume && !!drink.package_size && !!drink.cost_per_package) {
    retVal =
      ((drink.cost_per_package || 0) / ((drink.volume || 0) * (drink.package_size || 0))) * 1.19;
  }

  return retVal ? Math.round(retVal * 1000) / 1000 : retVal;
}

function calc_all_min_prices(drinks: Array<Drink>, min_prices: Array<number>) {
  drinks.forEach((drink) => {
    drink.volumes.forEach((volume) => {
      volume.min_prices = calc_min_prices(volume, drink.cost_per_volume, min_prices);
    });
  });
}

function helper(volume: DrinkPriceVolume, min_price: number) {
  let retVal = 0;
  let extraIngredientPrice = 0;
  volume.ingredients.forEach((ingredient) => {
    if (ingredient.drink_ingredient) {
      const _drink = usePricelistStore().drinks.find(
        (a) => a.id === ingredient.drink_ingredient?.ingredient_id
      );
      retVal += ingredient.drink_ingredient.volume * <number>(<unknown>_drink?.cost_per_volume);
    }
    if (ingredient.extra_ingredient) {
      extraIngredientPrice += ingredient.extra_ingredient.price;
    }
  });
  return (retVal * min_price) / 100 + extraIngredientPrice;
}

function calc_min_prices(
  volume: DrinkPriceVolume,
  cost_per_volume: number | undefined,
  min_prices: Array<number>
) {
  const retVal: Array<FG.MinPrices> = [];
  volume.min_prices = [];
  if (min_prices) {
    min_prices.forEach((min_price) => {
      let computedMinPrice: number;
      if (cost_per_volume) {
        computedMinPrice = (cost_per_volume * <number>volume.volume * min_price) / 100;
      } else {
        computedMinPrice = helper(volume, min_price);
      }
      retVal.push({ percentage: min_price, price: computedMinPrice });
    });
  }
  return retVal;
}

function clone<T>(o: T): T {
  return <T>JSON.parse(JSON.stringify(o));
}

interface DeleteObjects {
  prices: Array<FG.DrinkPrice>;
  volumes: Array<DrinkPriceVolume>;
  ingredients: Array<FG.Ingredient>;
}
export { DeleteObjects };

export { calc_volume, calc_cost_per_volume, calc_all_min_prices, calc_min_prices, clone };
